import pandas as pd
import statsmodels.api as sm
from sklearn.model_selection import train_test_split
from sklearn.metrics import mean_squared_error
import numpy as np

# Load the dataset and preprocess
data = pd.read_csv('c:/users/hjhjhj/desktop/shell.csv')
data['STATUS_BINARY'] = data['STATUS_CONF'].apply(lambda x: 1 if x == "validated" else 0)
data_dummies = pd.get_dummies(data.drop(columns=['STATUS_CONF', 'IP_INFO']), drop_first=True)
X = data_dummies.drop('STATUS_BINARY', axis=1)
y = data_dummies['STATUS_BINARY']
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# Adding a constant to the model (intercept)
X_train_const = sm.add_constant(X_train)
X_test_const = sm.add_constant(X_test)

# Identifying and removing highly correlated variables
correlation_matrix = X_train_const.corr()
high_corr_pairs = {tuple(sorted([column, index])): correlation_matrix[column][index]
                   for column in correlation_matrix.columns
                   for index in correlation_matrix.index
                   if column != index and abs(correlation_matrix[column][index]) >= 0.9}
columns_to_drop = [pair[1] for pair in high_corr_pairs.keys()]
X_train_reduced = X_train_const.drop(columns=columns_to_drop)
X_test_reduced = X_test_const.drop(columns=columns_to_drop)

# Convert all columns to float type
X_train_reduced_float = X_train_reduced.astype(float)
X_test_reduced_float = X_test_reduced.astype(float)

# Apply OLS regression on the float-typed dataset
ols_model_reduced_float = sm.OLS(y_train, X_train_reduced_float).fit()
ols_y_pred_reduced_float = ols_model_reduced_float.predict(X_test_reduced_float)
ols_mse_reduced_float = mean_squared_error(y_test, ols_y_pred_reduced_float)

# Print the MSE
print(f"MSE for the test set: {ols_mse_reduced_float:.6f}")
